\name{rSeqNP.Main}
\alias{rSeqNP.Main}
\title{
Testing differential expression and differential splicing of genes from RNA-seq data using non-parametric approaches
}
\description{
This is the main function of the package. It performs two types of tests. The first test is to identify DE genes and isoforms using standard non-parametric tests based on ranks of expression values of genes and isoforms.The second test is to jointly test differential expression and differential splicing of genes based on the permutation test of the gene differential score (GDS). See the reference for details of how the tests are performed.
}
\usage{
rSeqNP.Main(exp_object, type, outcome, gamma = NULL, nperm = 1e+06,
            seed = 100, exp.mean = 0.5, low.exp = 0.5)
}
\arguments{
  \item{exp_object}{
a list that is returned by the function \code{read.file}, which contains the following two data-frames:\cr
\code{iso.exp} is a data-frame that contains the expression values for the isoforms, with the 1st column as the gene names, the 2nd column as the isoform names, the 3rd column as the number of isoforms corresponding to the genes and the rest of the columns as the expression values (in RPKM or FPKM or other normalized unit) of the isoforms in each sample.\cr
\code{gene.exp} is a data-frame that contains the expression values for the genes, with the 1st column as the gene names, the 2nd column as the number of isoforms of the genes and the rest columns being the expression values(in RPKM or FPKM or other normalized unit) of the genes in each sample.
}
  \item{type}{
indicates the type of study design and outcomes. It should be one of the following: "\code{twoclass}" - unpaired two group comparison; "\code{twopaired}" - paired two group comparison; "\code{multiclass}" - multi-group comparison; "\code{quant}" - quantitative outcome data; "\code{survi}" - survival outcome data.
}
  \item{outcome}{
the outcome vector. For unpaired two group data, the first group must be denoted by 1, and the second group must be denoted by 2. For two paired group data, this vector should be like \code{c(1,2,1,2...,1,2)}, where the first two samples is a pair, and the third and fourth is a pair, and so on. For K class data, Class k must be denoted by k (k = 1,...,K). For quantitative outcome data, this vector is the quantitative outcome data (real numbers) corresponding to each sample. For survival outcome data, this vector is the survival time of each subject or the time at which the observation for the subject is censored.
}
  \item{gamma}{
optional, only required for survival outcome data, which is the censoring status vector for survival outcomes. 1 for observed (died) and 0 for censored. Default value is \code{NULL}.
}
  \item{nperm}{
number of permutations for the permuatation test besed on the GDS. Default value is 1e6. The user can set it to larger values, as larger values gives more stable results under different seed. But the computational time will also increases proportionally.
}
  \item{seed}{
random seed to generate the permutation indices. Note that different seeds usually give slightly different estimated p-values and FDR. Default value is 100.
}
  \item{exp.mean}{
the parameter to filter out the genes and isoforms with very low expressions. If the average expression levels of a gene or an isoform across all the samples is less than \code{exp.mean}, the gene or isoform will be filtered out. Default value is 0.5.
}
  \item{low.exp}{
this argument is used for two group comparison (paired or unpaired). If the average expression levels of a gene or an isoform across all the replicates in one group is less than \code{low.exp}, the gene or isoform will be given a lowly expressed label in the returned list (The returned list contains the results of tests) of this function. Default value is 0.5.
}
}
\details{
The estimated expression values for genes and isoforms, which are the input of this function (\code{exp_object}), can be obtained by rSeq, RSEM or Cuffdiff. The output files from rSeq that contain the expression values of genes and isoforms can be read by \code{read.file} function, and the returned list from \code{read.file} function can be used as the input for this function (\code{exp_object}). For the output files from RSEM and Cuffdiff, please format your input data as \code{exp_object}. 
}
\value{
The function will return a list containing the following three data-frames:
\item{isoform.DE.result }{this is the data-frame that contains the result from the test for isoform differential expression. The meanings of the column names are:\cr
\code{gene_name}: this is the name of the gene that contains the isoform.\cr
\code{isoform_name}: this is the name of the isoform tested.\cr
\code{isoform_number}: this is the number of isoforms corresponding to the gene in the analysis. Note it may be smaller than the number in the annotation database if you filter out those isoforms with low expressions.\cr
\code{WRS.stat}: this is the centered Wilcoxon rank-sum statistic. For paired two-group data, this column will be \code{WSR.stat} (centered Wilcoxon signed-rank statistic); for multi-group data, this column will be \code{KW.stat} (Kruskal-Wallis statistic); for quantitative outcome data, this column will be \code{SCC} (Spearman's correlation coefficient); for survival outcome data, this column will be \code{Score.stat} (Score statistic of the Cox proportional hazard model).\cr
\code{p.WRStest}: this is the p-values from the Wilcoxon rank-sum test for isoform DE. For paired two-group comparison, multi-group comparison, quantitative outcome data and survival outcome data, the corresponding tests will be Wilcoxon singed-rank test, Kruskal-Wallis test, the test of Spearman's rank correlation coefficient and the score test of the Cox proportional hazard model, respectively.\cr
\code{fdr.BH}: this is the false discovery rates (FDR) estimated by the Benjamini-Hochberg (BH) procedure.\cr
\code{Log.FC}: this column only exists for two group comparisons (paired orunpaired), which is the log2 of the average fold change between the two groups (Group 2 v.s. Group 1).\cr
\code{if.low}: this column only exists for two group comparisons (paired or unpaired), which is a Boolean data type. It is \code{TRUE} if the average isoform expression levels of all the replicates in one group are less than the value of \code{low.exp} specified in the \code{rSeqNP.Main} function (in this situation, the fold change can be either very large or very small, which is not a good summary statistic of the data). It is \code{FALSE} otherwise.\cr
}
\item{gene.DE.result }{this is the data-frame that contains the result from the test for gene differential expression. The meanings of the column names are:\cr
\code{gene.name}: this is the name of the gene tested.\cr
\code{WRS.stat}: this is the centered Wilcoxon rank-sum statistic. For paired two-group data, this column will be \code{WSR.stat} (centered Wilcoxon signed-rank statistic); for multi-group data, this column will be \code{KW.stat} (Kruskal-Wallis statistic); for quantitative outcome data, this column will be \code{SCC} (Spearman's correlation coefficient); for survival outcome data, this column will be \code{Score.stat} (Score statistic of the Cox proportional hazard model).\cr
\code{p.WRStest}: this is the p-values from the Wilcoxon rank-sum test for gene DE. For paired two-group comparison, multi-group comparison, quantitative outcome data and survival outcome data, the corresponding tests will be Wilcoxon singed-rank test, Kruskal-Wallis test, the test of Spearman's rank correlation coefficient and the score test of the Cox proportional hazard model, respectively.\cr
\code{fdr.BH}: this is the false discovery rates (FDR) estimated by the Benjamini-Hochberg (BH) procedure.\cr
\code{Log.FC}: this column only exists for two group comparisons (paired orunpaired), which is the log2 of the average fold change between the two groups (Group 2 v.s. Group 1).\cr
\code{if.low}: this column only exists for two group comparisons (paired or unpaired), which is a Boolean data type. It is \code{TRUE} if the average gene expression levels of all the replicates in one group are less than the value of \code{low.exp} specified in the \code{rSeqNP.Main} function (in this situation, the fold change can be either very large or very small, which is not a good summary statistic of the data). It is \code{FALSE} otherwise.\cr
}
\item{GDS.result }{this is the data-frame that contains the result from the permutation test based on the GDS. The meanings of the column names are:\cr
\code{gene.name}: this is the name of the gene tested.\cr
\code{GDS}: this is the gene differential score (GDS), which is the sum of squares of the non-parametric statistic used for testing isoform DE across all the isoforms of a gene. See the reference for details.\cr
\code{p.plugin}: this is the p-values from the permutation test based on the GDS, which is estimated by the permutation plug-in method. See the reference for details.\cr
\code{fdr.plugin}: this is the FDR from the permutation test based on the GDS, which is estimated by the permutation plug-in method. See the reference for details.\cr
\code{iso.num}: this is the number of isoforms that the gene have in the analysis. Note it may be smaller than that in the annotation database if you filter out those isoforms with low expression values.\cr
\code{T.value}: This is a statistic that measures the level of differential splicing for multi-isoform genes based on the L1-norm of the isoform expression levels. Large T values indicate high level of differential splicing. See the user manual of this package for details.\cr
}
}
\references{
Yang Shi, Arul M Chinnaiyan and Hui Jiang (2015). rSeqNP: A non-parametric approach for detecting differential expression and splicing from RNA-Seq data (in revision).
}
\author{Yang Shi and Hui Jiang}
\examples{
##This example is from simulated RNA-Seq data and it is for demonstration purpose only
##It takes about two minutes to run

##read the rSeq processed output files from a simulated RNA-Seq dataset
simulated_data_dir = system.file('extdata','rseq_data_simulated', package='rSeqNP')
setwd(simulated_data_dir)
mydata = read.rseq('samplelist_rseq.txt')

##invoke the main function
result = rSeqNP.Main(exp_object=mydata, type='twoclass', outcome=c(rep(1,10), rep(2,10)), nperm=1e5)

##check the result
result$isoform.DE.result[1:3,]
result$gene.DE.result[1:3,]
result$GDS.result[1:3,]
}
\keyword{ nonparametric }
\keyword{ permutation }
